#pragma once

//disable inheritance warning caused by multiple inheritance
#if _WIN32
#if _MSC_VER
#pragma warning(disable: 4250)
#endif
#endif

#include "OgreRenderingRun.h"
#include "ChildPSystemRenderingRun.h"
#include "OgreSharedRuns.h"


/**
	@brief ColorCubeMapRenderingRun used in an OGRE environment.
*/
class OgreChildPSystemRenderingRun : public OgreRenderingRun,
									public ChildPsystemRenderingRun
{	
public:

	/**
		@brief Constructor.

		@param sharedRuns			a pointer to the OgreSharedRuns this run belongs to
		@param name					the name of the texture to be created
		@param startFrame			adds an offset to the current frame number to help evenly distribute updates between frames
		@param updateInterval		update frequency
		@param materialName			the name of the material should be used when rendering the choton hit map
*/	
	OgreChildPSystemRenderingRun(OgreSharedRuns* sharedRuns,
								String name,
								unsigned long startFrame,
								unsigned long updateInterval,
								unsigned int resolution,
								bool perspectiveRendering,
								String childParticleSystemName,
								String particleScriptName,
								bool useOwnMaterial,
								String materialName
								);
	/**
		@brief returns the name of the resulting photon hit map
	*/
	String getImpostorTextureName(){return name;}
	
	bool canJoin(RenderingRun* run)
	{
		
		return false;
	}
	void setNode(SceneNode* n){psysNode = n;}
	Real getSmallSysRadius(){return sysRad;}

protected:	
	SceneNode* psysNode;
	bool perspectiveRendering;
	bool useOwnMaterial;
	String materialName;
	String childParticleSystemName;
	String particleScriptName;
	Real sysRad;
	/**
		@brief the created photon hit map texture
	*/
	Camera* impostorCamera;
	/**
		@brief a pointer to the OgreSharedRuns this run belongs to
	*/
	OgreSharedRuns* sharedRuns;
	/**
		@brief the name of the photonmap texture that was created by this run
	*/
	String name;
	/**
		@brief a pointer to the photonmap texture that was created by this run
	*/
	Texture* impostorTexture;
	

	//inherited
	void updateFrame(unsigned long frameNum);	
	//inherited
	inline void createImpostorTexture();

};
